/*****************************************************************************************
 * Cartesian.java

 * Version:           0.1

 * Description: 

 * Define Cartesian coordinates for individual atom in a protein struture.

 * authors:
 * initials    name            organization                email
 * ---------   --------------  ------------------------    ---------------------------
 *  LW         Lincong Wang   Dartmouth College           wlincong@cs.dartmouth.edu

 * history:
 * when        who    what
 * --------    ----   ----------------------------------------------------------
 * 06/25/04    LW     initial version for publication [Wang & Donald, J Biomol NMR 29: 223-242, 2004]

/**********************************************************************************************

 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.

 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.

 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 *             Contact Info:
 *                                                     Bruce Randall Donald
 *                                                     HB 6211
 *                                                     Dartmouth College
 *                                                     Hanover, NH 03755
 *                                                     brd@cs.dartmouth.edu
 *
 * If you use publish any results derived from the use of this program
 * please cite: Exact Solutions for Internuclear Vectors and Backbone
 * Dihedral Angles from NH Residual Dipolar Couplings in Two Media,
 * and Their Application in a Systematic Search Algorithm for
 * Determining Protein Backbone Structure. L. Wang and
 * B. R. Donald. Journal of Biomolecular NMR 29: 223-242 (2004)
 *
 *
 * Copyright (C) 2004  Lincong Wang and Bruce R. Donald
 *
 * <signature of Bruce Donald>, 25 June 2004 Bruce Donald, Professor of Computer Science

 *************************************************************************************************/

import java.io. *;
import java.util. *;

public class Cartesian{
    //the atom name and its three Cartesian coordinates
    private String atom;  
    private double xCoord;
    private double yCoord;
    private double zCoord;

    public Cartesian(){
	atom = null;
	xCoord = 0.0;
	yCoord = 0.0; 
	zCoord = 0.0;
    }
    public Cartesian(String aa, double X, double Y, double Z){
	atom = aa;
	xCoord = X;
	yCoord = Y;
	zCoord = Z;
    }

    public Cartesian(String aa, double [] c){
	atom = aa;
	xCoord = c[0];
	yCoord = c[1];
	zCoord = c[2];
    }

    //getting the individual values	
    public String getAtom(){
	return atom;
    }	
    public double getX(){
	return xCoord;
    }
    public double getY(){
	return yCoord;
    }
    public double getZ(){
	return zCoord;
    }

    //get all three coordinates
    public double[] getXYZ(){
	return (new double[]{xCoord,  yCoord,  zCoord});
    }
    public String toString(){
	String desc = atom +"  " + String.valueOf(xCoord) + "  " + String.valueOf(yCoord)
	    +" " + String.valueOf(zCoord);
	return desc;	
    }
}
