/*****************************************************************************************
 * Const.java

 * Version:           0.1

 * Description: 

 * Define various constants for the entire program.

 * authors:
 * initials    name            organization                email
 * ---------   --------------  ------------------------    ---------------------------
 *  LW         Lincong Wang   Dartmouth College           wlincong@cs.dartmouth.edu

 * history:
 * when        who    what
 * --------    ----   ----------------------------------------------------------
 * 06/25/04    LW     initial version for publication [Wang & Donald, J Biomol NMR 29: 223-242, 2004]

/**********************************************************************************************

 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.

 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.

 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 *             Contact Info:
 *                                                     Bruce Randall Donald
 *                                                     HB 6211
 *                                                     Dartmouth College
 *                                                     Hanover, NH 03755
 *                                                     brd@cs.dartmouth.edu
 *
 *  If you use publish any results derived from the use of this program
 *  please cite: Exact Solutions for Internuclear Vectors and Backbone
 *  Dihedral Angles from NH Residual Dipolar Couplings in Two Media,
 *  and Their Application in a Systematic Search Algorithm for
 *  Determining Protein Backbone Structure. L. Wang and
 *  B. R. Donald. Journal of Biomolecular NMR 29: 223-242 (2004)
 *
 *
 *  Copyright (C) 2004  Lincong Wang and Bruce R. Donald
 *
 *  <signature of Bruce Donald>, 25 June 2004 Bruce Donald, Professor of Computer Science

 *************************************************************************************************/

public class Const {
    static final double cst = Math.PI / 180.0; //conversion constant for angle
    static final double eps = 1.0E-10;         //for comparing two float numbers
    //For the definition of these theta angles see our paper
    static final double theta1 = 29.1406 * cst;  //Angles mined from 23 untraHigh resolution X-ray structures.
    static final double theta3 = -20.935 * cst;  //theta3 should be minus c.f. its definition
    static final double theta5 = 26.8237 * cst;
    static final double theta6 = -0.7518 * cst;
    static final double theta7 = 29.1044 * cst;
    static final double theta9 = -0.0021 * cst;
    static final double theta8 =  0.00204369051213495 * cst; //a magic angle needed for computing precisely coordinates 
    static final double[] sevenAngles = new double[]{theta1,theta9,theta3, theta5, theta6, theta7, theta8};
    static final double dN2CA  = 1.458;                     //bond lengths for backbone vectors
    static final double dCA2CO = 1.525;
    static final double dCO2N  = 1.329;
    static final double dN2H   = 1.020;
    static final double dCO2O  = 1.231;
    static final double dCA2HA = 1.090;
    static final double dN2COHBond   = 2.88;               //the distance across the H-bond
    static final double angleNCACOHA = 118.95951 * cst * 0.975; //angles for computing HA and CO atoms
    static final double angleNCAHA   = -19.4 * cst; 
    static final double angleCACOO   = 59.329 * cst;  

    //Matrices used for testing the degeneracy and computing orientations. 
    static Matrix mat = new Matrix(3,3);   
    static final double [][] signInv = {{-1.0, 0.0, 0.0},{0.0, -1.0, 0.0}, {0.0, 0.0, -1.0}}; //invert the coordinates
    static final double [][] xyInv   = {{-1.0, 0.0, 0.0},{0.0, -1.0, 0.0}, {0.0, 0.0,  1.0}}; //invert x,y direction
    static final double [][] xzInv   = {{-1.0, 0.0, 0.0},{0.0,  1.0, 0.0}, {0.0, 0.0, -1.0}}; //invert x,z direction
    static final double [][] yzInv   = {{ 1.0, 0.0, 0.0},{0.0, -1.0, 0.0}, {0.0, 0.0, -1.0}}; //invert y,z direction

    static final Matrix xyMat = new Matrix(xyInv);
    static final Matrix xzMat = new Matrix(xzInv);
    static final Matrix yzMat = new Matrix(yzInv);
    static Matrix[] mat4ThreeDirs = new Matrix[]{xyMat, xzMat, yzMat};
    static final Matrix signMat = new Matrix(signInv);
    static final Matrix r1x = mat.rotationMat(theta1, "+x"); 
    static final Matrix r9y = mat.rotationMat(theta9, "+y"); 
    static final Matrix r8z = mat.rotationMat(theta8, "+z"); 
    static final Matrix r3x = mat.rotationMat(theta3, "+x"); 
    static final Matrix r5x = mat.rotationMat(theta5, "+x");
    static final Matrix r6y = mat.rotationMat(theta6, "+y"); 
    static final Matrix r7x = mat.rotationMat(theta7, "+x");
    static final Matrix r7x6y5x = r7x.times(r6y.times(r5x));

    //Rotation Matrices for computing HA and O coordinates
    static final Matrix rHA1 = mat.rotationMat(angleNCACOHA,"+y");//theta_{41}=angleNCACOHA=PhiPsiCalPDB(caToCOVec,nToCAVec,caToHAVec);
    static final Matrix rHA2 = mat.rotationMat(angleNCAHA,  "+x");//theta_{42}=angleNCAHA=interAngle(nToCAVec, caToHAVec)-Math.PI/2;
    static final Matrix rHA2HA1 = rHA2.times(rHA1);
    static final Matrix rO   = mat.rotationMat(angleCACOO,  "+x");//angleCACOO = interAngle(caToCOVec, coToOVec);

    //Inverse matrices
    static final Matrix rHA1Inv = rHA1.transpose();
    static final Matrix rHA2Inv = rHA2.transpose();
    static final Matrix rHA2HA1Inv = rHA1Inv.times(rHA2Inv);
    static final Matrix rOInv   = rO.transpose();
    static final Matrix r1xInv  = r1x.transpose();
    static final Matrix r9yInv  = r9y.transpose();
    static final Matrix r1x9yInv  = r1xInv.times(r9yInv);
    static final Matrix r8zInv  = r8z.transpose();
    static final Matrix r3xInv  = r3x.transpose();
    static final Matrix r5xInv  = r5x.transpose();
    static final Matrix r6yInv  = r6y.transpose();
    static final Matrix r7xInv  = r7x.transpose();
    static final Matrix matL    = r1xInv.times(r9yInv);

    //Vectors for convenience and speed
    static final double [] coordCA = {0.0, dN2CA, 0.0};
    static final double [] coordNH = {0.0, 0.0, -dN2H};       
    static final double [] matRNH  = r5xInv.times(r6yInv.times(r7xInv.times(coordNH)));
    static final double [] matR = r5xInv.times(r6yInv.times(r7xInv.times(r8zInv.times(r1xInv.times(coordCA)))));
    static final double [] unitZ   = new double[]{0, 0, 1};  
    static final double [] dirCosCHcnt = rHA1Inv.times(rHA2Inv.times(unitZ));
    static final double [] unitMinusZ  = new double[]{0.0, 0.0, -1.0};
    static final double [] dirCosNHcnt = r5xInv.times(r6yInv.times(r7xInv.times(unitMinusZ)));
    static final double[][] arrLeftHand = new double[][]{{-1, 0, 0}, {0,1,0},{0,0,1}}; //for changing  handedness
    static final Matrix mLeftHand = new Matrix(arrLeftHand);

    //The average  Phi/Psi for a typical helix or strand
    static final double phiAveHelix =  -65.3 * cst;  
    static final double psiAveHelix =  -39.4 * cst;
    static final double phiAveBeta  = -120.0 * cst; 
    static final double psiAveBeta  =  138.0 * cst; 

    //The favorable Ramachandran (Ram) regions
    static final double phiHighHelix =  -30.0 * cst;  //helix region
    static final double phiLowHelix  = -100.0 * cst;
    static final double psiHighHelix =  -15.0 * cst;
    static final double psiLowHelix  =  -90.0 * cst;

    static final double phiHighBeta =  -70.0 * cst;  //Beta region
    static final double phiLowBeta  = -170.0 * cst;
    static final double psiHighBeta =  180.0 * cst;
    static final double psiLowBeta  =   80.0 * cst;
}
