/*****************************************************************************************
 * Root.java

 * Version:           0.1

 * Description: 

 * Represent the solutions for (NH) vector direction from (NH) RDC
 * measured in two different aligning media.

 * authors:
 * initials    name            organization                email
 * ---------   --------------  ------------------------    ---------------------------
 *  LW         Lincong Wang   Dartmouth College           wlincong@cs.dartmouth.edu

 * history:
 * when        who    what
 * --------    ----   ----------------------------------------------------------
 * 06/25/04    LW     initial version for publication [Wang & Donald, J Biomol NMR 29: 223-242, 2004]

/**********************************************************************************************

 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.

 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.

 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 *             Contact Info:
 *                                                     Bruce Randall Donald
 *                                                     HB 6211
 *                                                     Dartmouth College
 *                                                     Hanover, NH 03755
 *                                                     brd@cs.dartmouth.edu
 *
 * If you use publish any results derived from the use of this program
 * please cite: Exact Solutions for Internuclear Vectors and Backbone
 * Dihedral Angles from NH Residual Dipolar Couplings in Two Media,
 * and Their Application in a Systematic Search Algorithm for
 * Determining Protein Backbone Structure. L. Wang and
 * B. R. Donald. Journal of Biomolecular NMR 29: 223-242 (2004)
 *
 *
 * Copyright (C) 2004  Lincong Wang and Bruce R. Donald
 *
 * <signature of Bruce Donald>, 25 June 2004 Bruce Donald, Professor of Computer Science

 *************************************************************************************************/

import java.io. *;
import java.util. *;

public class Root{
    private int residueNo; 
    private Vector rootVec = null; 

    public Root(){
	residueNo = 0;
	rootVec = new Vector();
    }
    public Root(int n, Vector rVec){
	residueNo = n;
	rootVec = rVec;
    }

    //getting the values	
    public int getResidueNo(){
	return residueNo;
    }	
    public Vector getRootVec(){
	return rootVec;
    }
    public String toString(){
	String desc = String.valueOf(residueNo) +"\n";
	String str ="";
	double [] dirCos = new double[3];
	for (int j = 0;  j < rootVec.size(); j++){
	    dirCos = (double[] )rootVec.elementAt(j);
	    str += String.valueOf(dirCos[0])+" "+String.valueOf(dirCos[1])+"  "+String.valueOf(dirCos[2])+"\n";
	}
	return (desc+str);	
    }
}
